// SWFile.java
// used to read in only old style (from DOS version of signwriter) sgn files.
// Version 1.0 by DTAI
// Version 1.1 by Larry S. Peranich  12-17-98
//      Change so that symbol codes are translated from the codes used in
//      the old symbol file, to the codes used in the new symbol file.  The
//      symbol file was changed to put the symbols in Sign-Symbol-Sequence,
//      which is an "alphabetical order" for sign symbols.
// Version 1.2 by Larry S. Peranich  12-27-98
//      Move the symbol code translation to SignSymbol.java because the
//      dictionary code needs the same translations, so this code is
//      back to about the same as before, with some additional error checking.

package signwriting;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * reads the symbol file
 */
public class SWFile {

    public static final int CHAR_CODE = 250;	       // ASCII char sym code	 *)
    public static final int AHead    = 251;	       // arrowhead sym code   	 *)
    public static final int Cursor   = 251;	       // cursor sym code	 *)
    public static final int Marker   = 252;	       // select marker sym code *)
    public static final int Spacer   = 252;	       // intersign space code   *)

    public static final int NEW_SIGN  = 253;	       // control codes *)
    public static final int NEW_LINE  = 254;
    public static final int EOF      = 255;

    public static final int SP_WIDTH = 36; // pixel width of a space (seems like a lot!)

    private static final int MAXBUF = 29696;
    

    private String filename;

    /**
     * constructor, takes a symbol file name, opens it and reads it
     */
    public SWFile(String filename) {
        this.filename = filename;
    }

    public DocumentPart[] readDocument() throws IOException {
        DataInputStream in = new DataInputStream(
                             new BufferedInputStream(
                             new FileInputStream(filename)));
		int start = in.available();
		byte version = in.readByte(); // 4 == 4.0, 5 == 4.1
		byte host = in.readByte(); // 3 == PC/DOS
		in.readByte(); // skip a byte
		short mrk1 = PCNumbers.makeShort(in.read(),in.read());
		short mrk2 = PCNumbers.makeShort(in.read(),in.read());
		short len = PCNumbers.makeShort(in.read(),in.read());

		in.skip(512-(start-in.available())); // skip remainder of 512 byte header

		if (len > MAXBUF) {
		    throw new IOException("bad length "+len+" greater than old MAXBUF="+MAXBUF);
		}

		len -= 2; // for some reason they had done this in the code, but it don't make sense.

        boolean fmarks = (mrk1 != -1) && (mrk2 != -1);

        if (fmarks) {
            len = (short)(mrk2 - mrk1);
        }

        if (len < 0) {
		    throw new IOException("bad length "+len);
		}

		DocumentPart[] docParts = new DocumentPart[1];
		int docPartIndex = 0;
		int code = in.read();
		int nextSignPos = 0;
		while(code != EOF) {
		    if (docPartIndex >= docParts.length) {
		        DocumentPart[] newarray = new DocumentPart[docParts.length*2];
		        System.arraycopy(docParts,0,newarray,0,docParts.length);
		        docParts = newarray;
		    }
		    if (docPartIndex == 0 && code == NEW_LINE) {
                nextSignPos = in.read();
		    } else {
    		    if (code == NEW_SIGN) {
                    Sign sign = getSign(docParts, docPartIndex);
                    sign.setNextSignPos(nextSignPos*2 + SP_WIDTH);
                    nextSignPos = in.read();
    		        docPartIndex++;
    		    } else if (code == NEW_LINE) {
    		        if (docParts[docPartIndex] != null) {
        		        docPartIndex++;
    		        }
        		    if (docPartIndex >= docParts.length) {
        		        DocumentPart[] newarray = new DocumentPart[docParts.length*2];
        		        System.arraycopy(docParts,0,newarray,0,docParts.length);
        		        docParts = newarray;
        		    }
    		        docParts[docPartIndex++] = DocumentPart.NEW_LINE;
                    nextSignPos = in.read();
    		    } else {
                    Sign sign = getSign(docParts, docPartIndex);
    		        if (code == CHAR_CODE) {
            		    sign.add(new SignCharacter(in.readByte(),in.readByte(),in.readByte()));
        		    } else {
        		        try {
        		            // make sure old code is legal, then map to new code
        		            if (code < 0 || code > 249) throw new
        		                    ArrayIndexOutOfBoundsException(
        		                    "Bad code while reading old style sgn file");
               		    sign.add(new SignSymbol(code,in.read(),
                		            in.readByte(),in.readByte(),true));
                		} catch (ArrayIndexOutOfBoundsException aioobe) {
                		    aioobe.printStackTrace();
                		    System.err.println("probably arrowhead out of range for now?");
                		}
        		    }
        		}
    		}
		    code = in.read();
		}

        in.close();

        return docParts;
    }

    static Sign getSign(DocumentPart[] docParts, int docPartIndex) {
        if (docParts[docPartIndex] == null) {
	        docParts[docPartIndex] = new Sign();
        }
        return (Sign)docParts[docPartIndex];
    }
}


